@@ -456,7 +456,8 @@ class GitClient(VcsClientBase):
         return response
 
     def get_affected_files(self, revision):
-        cmd = "git show {0} --pretty='format:' --name-only".format(
+        # Making changes for windows support
+        cmd = "git show {0} --pretty=format: --name-only".format(
             revision)
         code, output, _ = run_shell_command(cmd, shell=True, cwd=self._path)
         affected = []
@@ -495,7 +496,8 @@ class GitClient(VcsClientBase):
 
         return response
 
-    def get_status(self, basepath=None, untracked=False):
+    def get_status(self, basepath=None, untracked=False, porcelain=False):
+        status_flag = '--porcelain' if porcelain else '-s'
         response = None
         if basepath is None:
             basepath = self._path
@@ -503,7 +505,7 @@ class GitClient(VcsClientBase):
             rel_path = normalized_rel_path(self._path, basepath)
             # git command only works inside repo
             # self._path is safe against command injection, as long as we check path.exists
-            command = "git status -s "
+            command = "git status {0} ".format(status_flag)
             if not untracked:
                 command += " -uno"
             _, response, _ = run_shell_command(command,
@@ -517,9 +519,9 @@ class GitClient(VcsClientBase):
                                                          rel_path,
                                                          line[3:])
             if LooseVersion(self.gitversion) > LooseVersion('1.7'):
-                command = "git submodule foreach --recursive git status -s"
                 if not untracked:
-                    command += " -uno"
+                    status_flag += " -uno"
+                command = "git submodule foreach --recursive 'git status {0}'".format(status_flag)
                 _, response2, _ = run_shell_command(command,
                                                     shell=True,
                                                     cwd=self._path)
@@ -739,10 +741,27 @@ class GitClient(VcsClientBase):
         if not self.detect_presence():
             return False
 
+        # Export should work regardless of the state of the local dir (might have modified files),
+        # so we clone to a temporary folder and checkout the specified version there.
+        # If the repo has submodules with relative URLs, cloning to a temp dir doesn't work.
+        # To support this case, first check if current version is already checked out and there are no
+        # modifications and try to export from current local dir.
+        # see https://github.com/vcstools/vcstools/pull/130
+        current_sha = self.get_version()
+        export_sha = self.get_version(version)
+        if current_sha == export_sha and self.get_diff() == '':
+            archiver = GitArchiver(main_repo_abspath=self.get_path(), force_sub=True)
+            filepath = '{0}.tar.gz'.format(basepath)
+            archiver.create(filepath)
+            return filepath
+
+        # export a different version than currently checked out or there are local changes
         try:
             # since version may relate to remote branch / tag we do not
             # know about yet, do fetch if not already done
             self._do_fetch()
+            # export should work regardless of the state of the local dir (might have modified files)
+            # so we clone to a temporary folder and checkout the specified version there
             tmpd_path = tempfile.mkdtemp()
             try:
                 tmpgit = GitClient(tmpd_path)
@@ -826,12 +845,20 @@ class GitClient(VcsClientBase):
                 return True
             print("Cannot fast-forward, local repository and remote '%s' have diverged." % branch_parent)
             return False
+        # 'git reset --keep' doesn't refresh the index. Do it manually to avoid
+        # errors as reported in: https://github.com/vcstools/wstool/issues/77
+        cmd = "git update-index -q --refresh"
+        run_shell_command(cmd,
+                          shell=True,
+                          cwd=self._path,
+                          show_stdout=False,
+                          verbose=verbose)
         if verbose:
             print("Rebasing repository")
         # Rebase, do not pull, because somebody could have
         # commited in the meantime.
         if LooseVersion(self.gitversion) >= LooseVersion('1.7.1'):
-            # --keep allows o rebase even with local changes, as long as
+            # --keep allows to rebase even with local changes, as long as
             # local changes are not in files that change between versions
             cmd = "git reset --keep remotes/%s/%s" % (default_remote, branch_parent)
             value, _, _ = run_shell_command(cmd,
